#! /usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addValidatorInstructionBuilder = void 0;
require('dotenv').config();
const fs = require('fs');
const commander_1 = require("commander");
const anchor_1 = require("@coral-xyz/anchor");
const _1 = require(".");
const web3_js_1 = require("@solana/web3.js");
const __path = process.cwd();
const figlet = require('figlet');
const MAINNET_RPC = 'https://api-mainnet-beta.renec.foundation:8899/';
const TESTNET_RPC = 'https://api-testnet.renec.foundation:8899/';
const LOCAL_RPC = 'http://localhost:8899/';
console.log(figlet.textSync('Liquid Staking'));
console.log('');
const ownerKey = JSON.parse(fs.readFileSync(__path + '/sdk/src/.wallets/owner.json'));
const program = new commander_1.Command();
function sleep(ms) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise(resolve => setTimeout(resolve, ms));
    });
}
program
    .command('initialize')
    .description('initialize a new liquid staking.')
    .option('-p, --program-id <string>', 'Program ID')
    .option('-n, --network <string>', 'Network: mainnet, testnet, localnet', 'mainnet')
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    const { network, programId } = params;
    console.log('Initializing a new liquid staking.', params);
    if (!programId) {
        console.log('--program_id is not found ');
        return;
    }
    const PROGRAM_ID = new web3_js_1.PublicKey(programId);
    let validatorVote = '4EPZEue64UoFUHc1TpMdwCPYb4p33ndiRDVnunaZG86z';
    let rpc = MAINNET_RPC;
    if (network === 'testnet') {
        rpc = TESTNET_RPC;
        validatorVote = 'F8kbmnnoXuX5GwG9Vx8YYpvkDGqczkccL4NVN3EWnTv5';
    }
    if (network === 'localnet') {
        rpc = LOCAL_RPC;
        validatorVote = 'F8kbmnnoXuX5GwG9Vx8YYpvkDGqczkccL4NVN3EWnTv5';
    }
    const CONNECTION = new _1.web3.Connection(rpc, {
        commitment: 'confirmed',
    });
    const SDK_USER = _1.web3.Keypair.fromSecretKey(new Uint8Array(ownerKey));
    const PROVIDER = new anchor_1.AnchorProvider(CONNECTION, new _1.Wallet(SDK_USER), {
        commitment: 'confirmed' /*, skipPreflight: true*/,
    });
    const config = new _1.MarinadeConfig({
        connection: CONNECTION,
        publicKey: SDK_USER.publicKey,
        marinadeFinanceProgramId: PROGRAM_ID,
    });
    const marinade = new _1.Marinade(config);
    const minStake = new anchor_1.BN(1000000000);
    const slotsForStakeDelta = new anchor_1.BN(3000);
    const additionalSpace = 32;
    const lpLiquidityTarget = new anchor_1.BN('50000000000000');
    const transactions = yield marinade.initialize(minStake, slotsForStakeDelta, lpLiquidityTarget, additionalSpace);
    let marinadeState = undefined;
    for (let i = 0; i < transactions.length; i++) {
        const transaction = transactions[i];
        const transactionSignature = yield PROVIDER.sendAndConfirm(transaction.transaction, [SDK_USER, ...transaction.singers]);
        if (transaction.label === 'createState') {
            marinadeState = transaction.singers[0].publicKey;
            console.log('State account created:', transaction.singers[0].publicKey.toBase58());
        }
        console.log(transaction.label, transactionSignature, transaction.transaction.instructions.length);
    }
    if (marinadeState) {
        //   await sleep(1000)
        const newConfig = new _1.MarinadeConfig({
            connection: CONNECTION,
            publicKey: SDK_USER.publicKey,
            marinadeFinanceProgramId: PROGRAM_ID,
            marinadeStateAddress: marinadeState,
        });
        const newMarinade = new _1.Marinade(newConfig);
        const validatorVotePubkey = new _1.web3.PublicKey(validatorVote);
        const addIx = yield addValidatorInstructionBuilder({
            marinade: newMarinade,
            validatorScore: 1000,
            validatorVote: validatorVotePubkey,
            rentPayer: SDK_USER.publicKey,
        });
        const addTx = new _1.web3.Transaction().add(addIx);
        const txid = yield PROVIDER.sendAndConfirm(addTx, [SDK_USER]);
        console.log('add validator', txid);
    }
    else {
        console.log('State account not created');
    }
}));
program
    .command('addLiquidity')
    .description('Add liquidity to the liquid staking.')
    .option('-a, --amount <string>', 'Amount of liquidity to add')
    .option('-n, --network <string>', 'Network: mainnet, testnet, localnet', 'mainnet')
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, amount } = params;
    console.log('Add liquidity to the liquid staking.', params);
    if (!amount) {
        console.log('--amount is not found ');
        return;
    }
    let PROGRAM_ID = new web3_js_1.PublicKey('Xv4P2xKrRaQfPFU1heTXsbmmiafrExaSZou339ujcUW');
    let marinadeStateAddress = new web3_js_1.PublicKey('6dgKgb8GyUGAoyzUP1qdnLhQRYJqBFfR6TwVFsgNEC3H');
    let rpc = MAINNET_RPC;
    if (network === 'testnet') {
        rpc = TESTNET_RPC;
        PROGRAM_ID = new web3_js_1.PublicKey('CAfSCvCw8ewGvtnZcR7HFJ6K3ZdAFZDt6cFyABYneq8s');
        marinadeStateAddress = new web3_js_1.PublicKey('6TejU7oSRXbTaXEsVRHXeMRu2JhDPfCD1gJQ6ZrPYBjK');
    }
    if (network === 'localnet') {
        rpc = LOCAL_RPC;
        PROGRAM_ID = new web3_js_1.PublicKey('CAfSCvCw8ewGvtnZcR7HFJ6K3ZdAFZDt6cFyABYneq8s');
        marinadeStateAddress = new web3_js_1.PublicKey('6TejU7oSRXbTaXEsVRHXeMRu2JhDPfCD1gJQ6ZrPYBjK');
    }
    const CONNECTION = new _1.web3.Connection(rpc, {
        commitment: 'confirmed',
    });
    const SDK_USER = _1.web3.Keypair.fromSecretKey(new Uint8Array(ownerKey));
    const PROVIDER = new anchor_1.AnchorProvider(CONNECTION, new _1.Wallet(SDK_USER), {
        commitment: 'confirmed' /*, skipPreflight: true*/,
    });
    const newConfig = new _1.MarinadeConfig({
        connection: CONNECTION,
        publicKey: SDK_USER.publicKey,
        marinadeFinanceProgramId: PROGRAM_ID,
        marinadeStateAddress,
    });
    const marinade = new _1.Marinade(newConfig);
    const { transaction } = yield marinade.addLiquidity(_1.MarinadeUtils.solToLamports(Number(amount)));
    const txid = yield PROVIDER.sendAndConfirm(transaction, [SDK_USER]);
    console.log('addLiquidity txid:', txid);
}));
program
    .command('reallocateStakeList')
    .description('Reallocate stake list.')
    .option('-num, --numStakeItems <string>', 'Number of stake items')
    .option('-n, --network <string>', 'Network: mainnet, testnet, localnet', 'mainnet')
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, numStakeItems } = params;
    console.log('Reallocate stake list.', params);
    if (!numStakeItems) {
        console.log('--numStakeItems is not found ');
        return;
    }
    let PROGRAM_ID = new web3_js_1.PublicKey('Xv4P2xKrRaQfPFU1heTXsbmmiafrExaSZou339ujcUW');
    let marinadeStateAddress = new web3_js_1.PublicKey('6dgKgb8GyUGAoyzUP1qdnLhQRYJqBFfR6TwVFsgNEC3H');
    let rpc = MAINNET_RPC;
    if (network === 'testnet') {
        rpc = TESTNET_RPC;
        PROGRAM_ID = new web3_js_1.PublicKey('CAfSCvCw8ewGvtnZcR7HFJ6K3ZdAFZDt6cFyABYneq8s');
        marinadeStateAddress = new web3_js_1.PublicKey('6TejU7oSRXbTaXEsVRHXeMRu2JhDPfCD1gJQ6ZrPYBjK');
    }
    if (network === 'localnet') {
        rpc = LOCAL_RPC;
        PROGRAM_ID = new web3_js_1.PublicKey('CAfSCvCw8ewGvtnZcR7HFJ6K3ZdAFZDt6cFyABYneq8s');
        marinadeStateAddress = new web3_js_1.PublicKey('6TejU7oSRXbTaXEsVRHXeMRu2JhDPfCD1gJQ6ZrPYBjK');
    }
    const CONNECTION = new _1.web3.Connection(rpc, {
        commitment: 'confirmed',
    });
    const SDK_USER = _1.web3.Keypair.fromSecretKey(new Uint8Array(ownerKey));
    const PROVIDER = new anchor_1.AnchorProvider(CONNECTION, new _1.Wallet(SDK_USER), {
        commitment: 'confirmed' /*, skipPreflight: true*/,
    });
    const newConfig = new _1.MarinadeConfig({
        connection: CONNECTION,
        publicKey: SDK_USER.publicKey,
        marinadeFinanceProgramId: PROGRAM_ID,
        marinadeStateAddress,
    });
    const marinade = new _1.Marinade(newConfig);
    const { transaction } = yield marinade.reallocStakeList(Number(numStakeItems));
    const txid = yield PROVIDER.sendAndConfirm(transaction, [SDK_USER]);
    console.log('reallocateStakeList txid:', txid);
}));
function addValidatorInstructionBuilder({ marinade, validatorScore, validatorVote, rentPayer, }) {
    return __awaiter(this, void 0, void 0, function* () {
        const marinadeState = yield marinade.getMarinadeState();
        console.log('validatorList', marinadeState.state.validatorSystem.validatorList.account.toBase58());
        console.log('state', marinade.config.marinadeStateAddress.toBase58());
        return yield marinade.marinadeFinanceProgram.program.methods
            .addValidator(validatorScore)
            .accountsStrict({
            state: marinadeState.marinadeStateAddress,
            validatorList: marinadeState.state.validatorSystem.validatorList.account,
            rentPayer,
            rent: _1.web3.SYSVAR_RENT_PUBKEY,
            validatorVote,
            managerAuthority: marinadeState.state.validatorSystem.managerAuthority,
            duplicationFlag: yield marinadeState.validatorDuplicationFlag(validatorVote),
            clock: _1.web3.SYSVAR_CLOCK_PUBKEY,
            systemProgram: _1.web3.SystemProgram.programId,
        })
            .instruction();
    });
}
exports.addValidatorInstructionBuilder = addValidatorInstructionBuilder;
program.parse([]);
//# sourceMappingURL=cli.js.map